@@ -1,17 +1,55 @@
-"""
-@Author  : Likianta <likianta@foxmail.com>
-@Module  : excel_writer.py
-@Created : 2018-00-00
-@Updated : 2020-11-22
-@Version : 2.3.13
-@Desc    : ExcelWriter is a post-packing implementation based on XlsxWriter.
-"""
-from warnings import filterwarnings
+import typing as _t
+from collections import defaultdict
 
-from ._typing import ExcelWriterHint as Hint
+import xlsxwriter
 
-# shield Python 3.8 SyntaxWarning from XlsxWriter.
-filterwarnings('ignore', category=SyntaxWarning)
+
+class T:
+    from xlsxwriter.workbook import Workbook as _Workbook
+    from xlsxwriter.workbook import Worksheet as _Worksheet
+    
+    Dict = _t.Dict
+    Optional = _t.Optional
+    Union = _t.Union
+    
+    Rowx = int
+    Colx = int
+    Cell = _t.Tuple[Rowx, Colx]
+    
+    class _CellFormat(_t.TypedDict):
+        """
+        References:
+            xlsxwriter.format.Format
+            https://xlsxwriter.readthedocs.io/format.html
+        """
+        # xlsxwriter.format.Format
+        align: _t.Literal['left', 'right', 'center']
+        valign: _t.Literal['top', 'bottom', 'vcenter']
+        textwrap: bool
+        num_percent: str
+        # TODO: more...
+    
+    CellFormat = _t.Union[None, _t.Dict[str, str], _CellFormat]
+    
+    CellValue = _t.Union[None, bool, float, int, str]
+    RowValues = _t.Iterable[CellValue]
+    ColValues = _t.Iterable[CellValue]
+    Header = _t.List[CellValue]
+    
+    RowsValues = _t.List[RowValues]
+    ColsValues = _t.List[ColValues]
+    
+    WorkBook = _Workbook
+    WorkSheet = _Worksheet
+    Sheetx = int
+    SheetName = _t.Optional[str]
+    
+    class SheetInfo(_t.TypedDict):
+        sheet_name: _t.Optional[str]
+        sheetx: int
+        rowx: int
+    
+    SheetManager = _t.Dict[Sheetx, SheetInfo]
 
 
 class ExcelWriter:
@@ -19,54 +57,43 @@ class ExcelWriter:
         designed to provide more flexible and ease of use excel writing
         experience.
     
-    Refer: https://www.jianshu.com/p/187e6b86e1d9
-    Palette: `xlsxwriter.format.Format._get_color`
+    References:
+        https://www.jianshu.com/p/187e6b86e1d9
+        Palette: `xlsxwriter.format.Format._get_color`
     """
-    __h: str
+    book: T.WorkBook
+    filepath: str
+    rowx: T.Rowx  # auto increasing row index, see self.writeln, self.writelnx.
+    sheet: T.WorkSheet
     
     _is_constant_memory: bool
-    _merge_format: Hint.CellFormat
-    _sheet_mgr: Hint.SheetManager
+    _merge_format: T.CellFormat
+    _sheet_mgr: '_SheetManager'
     
-    book: Hint.WorkBook
-    filepath: str
-    rowx: int  # auto increasing row index, see self.writeln, self.writelnx.
-    sheet: Hint.WorkSheet
-    sheetx: int
+    __h: str
     
-    def __init__(self, filepath: str, sheet_name: Hint.SheetName = '',
+    def __init__(self, filepath: str, sheet_name: T.SheetName = 'sheet 1',
                  **options):
         """
-        :param filepath: a filepath ends with '.xlsx' ('.xls' is not supported).
-        :param sheet_name: Union[str, None].
-            str: If string is empty, will create a sheet with default name --
-                'sheet 1'.
-            None: `None` is a special token, it tells ExcelWriter NOT to create
-                sheet in `__init__` stage.
-        :param options: workbook format.
+        Args:
+            filepath: a filepath ends with '.xlsx' ('.xls' is not allowed)
+            sheet_name:
+                str:
+                None: `None` is a special token, it tells ExcelWriter NOT to 
+                    create sheet in `__init__` stage.
+            **options: workbook format.
         """
         if not filepath.endswith('.xlsx'):
-            raise ValueError('ExcelWriter only supports .xlsx file type.',
-                             filepath)
-        
-        self.__h = 'parent'  # just for adjusting lk_logger's hierarchy in
-        #   `self.__exit__` and `self.save`.
-        
-        self._is_constant_memory = options.get('constant_memory', False)
-        self._sheet_mgr = {'sheets': [], 0: {}}
-        #   注: xlsxwriter.Workbook 已提供 sheet_names 的访问, 所以 self._sheet_mgr
-        #   ['sheets'] 其实是没有必要的. 我会在未来调整 (简化) self._sheet_mgr.
-        
-        self.sheetx = 0
-        self.rowx = 0
-        
+            raise Exception(
+                'ExcelWriter only supports .xlsx file type.', filepath
+            )
         self.filepath = filepath
         
         # create workbook
-        import xlsxwriter
+        self._is_constant_memory = options.get('constant_memory', False)
         self.book = xlsxwriter.Workbook(
-            filename=filepath,
-            options={  # the default options
+            filename=self.filepath,
+            options=options or {  # the default options
                 'strings_to_numbers'       : True,
                 'strings_to_urls'          : False,
                 'constant_memory'          : False,
@@ -78,52 +105,26 @@ class ExcelWriter:
                     # 'align': 'left',  # cell's horizontal alignment
                     # 'valign': 'vcenter',  # cell's vertical alignment
                     # 'text_wrap': False,  # auto line wrap (default False)
-                },
-                **options
+                }
             }
         )
         
+        self._sheet_mgr = _SheetManager()
         # create sheet (if `sheet_name` is not None)
         if sheet_name is not None:
             self.add_new_sheet(sheet_name)
         
+        self.rowx = 0  # see `self.writeln:docs:notes`
+        
         # the format for merge range
-        self._merge_format = self.book.add_format({
+        self._merge_format = self.add_format({
             'align' : 'center',
             'valign': 'vcenter',
             # 'text_wrap': False,  # auto line wrap (default False)
-        })  # REF: https://blog.csdn.net/lockey23/article/details/81004249
-    
-    def add_new_sheet(self, sheet_name=''):
-        # Save current sheet info
-        self._sheet_mgr[self.sheetx].update({
-            'sheetx': self.sheetx,
-            'rowx': self.rowx,
-        })
+        })  # ref: https://blog.csdn.net/lockey23/article/details/81004249
         
-        self.sheetx += 1
-        self.rowx = 0
-        if not sheet_name:  # auto create sheet name
-            sheet_name = f'sheet {self.sheetx}'  # -> 'sheet 1', 'sheet 2', ...
-        self.sheet = self.book.add_worksheet(sheet_name)
-        
-        # Add new sheet info
-        self._sheet_mgr['sheets'].append(sheet_name)
-        self._sheet_mgr[self.sheetx] = {
-            'sheet_name': sheet_name,
-            'sheetx'    : self.sheetx,
-            'rowx'      : self.rowx,
-        }
-        
-    def activate_sheet(self, sheetx: Hint.Sheetx):
-        if isinstance(sheetx, int):
-            self.sheetx = sheetx
-            sheet_name = self._sheet_mgr['sheets'][sheetx]
-        else:
-            self.sheetx = self._sheet_mgr['sheets'].index(sheetx)
-            sheet_name = sheetx
-        self.sheet = self.book.get_worksheet_by_name(sheet_name)
-        self.rowx = self._sheet_mgr[sheetx]['rowx']
+        self.__h = 'parent'  # just for adjusting lk_logger's hierarchy in
+        #   `self.__exit__` and `self.save`.
     
     def __enter__(self):
         """ Return self to use with `with` statement.
@@ -139,8 +140,69 @@ class ExcelWriter:
     
     # --------------------------------------------------------------------------
     
-    def write(self, rowx: Hint.Rowx, colx: Hint.Colx, data: Hint.CellValue,
-              fmt=None):
+    def add_format(self, fmt: T.CellFormat) -> T.CellFormat:
+        if fmt is None or fmt is {}:
+            return None
+        elif isinstance(fmt, dict):
+            return self.book.add_format(fmt)
+        else:
+            return fmt
+    
+    _convert_format = add_format
+    
+    def add_new_sheet(self, sheet_name=''):
+        # save old sheet info
+        self._sheet_mgr.update_info(self.sheetx, {
+            'sheet_name': self.sheet_name,
+            'sheetx'    : self.sheetx,
+            'rowx'      : self.rowx,
+        })
+        
+        new_sheetx = self._sheet_mgr.add_new_sheet()
+        if sheet_name:
+            assert sheet_name not in self._sheet_mgr.sheet_by_name, (
+                f'The sheet ("{sheet_name}") has already existed'
+            )
+        else:
+            sheet_name = f'sheet {new_sheetx + 1}'
+            #   'sheet 1', 'sheet 2', ...
+        self._sheet_mgr.activate(new_sheetx)
+        
+        self.sheet = self.book.add_worksheet(sheet_name)
+        
+        self.rowx = 0
+        
+        # add new sheet info
+        self._sheet_mgr.update_info(self.sheetx, {
+            'sheet_name': sheet_name,
+            'sheetx'    : self.sheetx,
+            'rowx'      : self.rowx,
+        })
+    
+    def activate_sheet(self, x: T.Union[T.Sheetx, T.SheetName]):
+        if isinstance(x, int):
+            sheetx = x
+            sheet_name = self._sheet_mgr.sheet_by_index[x]
+        else:
+            sheet_name = x
+            sheetx = self._sheet_mgr.sheet_by_name[sheet_name]
+        
+        self._sheet_mgr.activate(sheetx)
+        self.sheet = self.book.get_worksheet_by_name(sheet_name)
+        self.rowx = self._sheet_mgr.sheet_info[sheetx]['rowx']
+    
+    @property
+    def sheetx(self) -> T.Sheetx:
+        return self._sheet_mgr.current_index
+    
+    @property
+    def sheet_name(self) -> T.SheetName:
+        return self._sheet_mgr.current_name
+    
+    # --------------------------------------------------------------------------
+    
+    def write(self, rowx: T.Rowx, colx: T.Colx, data: T.CellValue,
+              fmt: T.CellFormat = None):
         """ Write data to cell.
         
         :param rowx: int. Row number, starts from 0.
@@ -148,39 +210,44 @@ class ExcelWriter:
         :param data: Union[str, int, float, bool, None]
         :param fmt: Union[None, Dict]
         """
-        self.sheet.write(rowx, colx, data, fmt)
+        self.sheet.write(rowx, colx, data, self._convert_format(fmt))
     
-    def writeln(self, *row: Hint.CellValue, auto_index=False,
-                purify_values=False, fmt=None):
-        """ Write line of data to cells (with auto line breaks). """
+    def writeln(self, *row: T.CellValue, auto_index=False,
+                purify_values=False, fmt: T.CellFormat = None):
+        """ Write line of data to cells (with auto line breaks).
+        
+        Notes:
+            `self.rowx` 须保证在调用本方法之前, 处于空行的开始位置; 调用本方法之
+            后, 更新到下一空行的开始位置.
+        """
         if purify_values:
             row = self.purify_values(row)
         if self.rowx == 0 or not auto_index:
-            self.sheet.write_row(self.rowx, 0, row, fmt)
+            self.sheet.write_row(self.rowx, 0, row, self._convert_format(fmt))
         else:
-            self.sheet.write(self.rowx, 0, self.rowx, fmt)
-            self.sheet.write_row(self.rowx, 1, row, fmt)
+            self.sheet.write(self.rowx, 0, self.rowx, self._convert_format(fmt))
+            self.sheet.write_row(self.rowx, 1, row, self._convert_format(fmt))
         self.rowx += 1
     
-    def writelnx(self, *row: Hint.CellValue, fmt=None):
-        self.writeln(*row, auto_index=True, fmt=fmt)
+    def writelnx(self, *row: T.CellValue, fmt: T.CellFormat = None):
+        self.writeln(*row, auto_index=True, fmt=self._convert_format(fmt))
     
-    def writerow(self, rowx: int, data: Hint.RowValues, offset=0,
-                 purify_values=False, fmt=None):
+    def writerow(self, rowx: int, data: T.RowValues, offset=0,
+                 purify_values=False, fmt: T.CellFormat = None):
         """ Write row of data to cells. """
         if purify_values:
             data = self.purify_values(data)
-        self.sheet.write_row(rowx, offset, data, fmt)
+        self.sheet.write_row(rowx, offset, data, self._convert_format(fmt))
     
-    def writecol(self, colx: int, data: Hint.ColValues, offset=0,
-                 purify_values=False, fmt=None):
+    def writecol(self, colx: int, data: T.ColValues, offset=0,
+                 purify_values=False, fmt: T.CellFormat = None):
         """ Write column of data to cells. """
         if purify_values:
             data = self.purify_values(data)
-        self.sheet.write_column(offset, colx, data, fmt)
+        self.sheet.write_column(offset, colx, data, self._convert_format(fmt))
     
     @staticmethod
-    def purify_values(row: Hint.RowValues):
+    def purify_values(row: T.RowValues):
         """
         Callers: self.writeln, self.writerow, self.writecol.
         """
@@ -194,8 +261,8 @@ class ExcelWriter:
     
     # --------------------------------------------------------------------------
     
-    def merging_visual(self, rows: Hint.RowValues, to_left='<', to_up='^',
-                       offset=(0, 0), fmt=None):
+    def merging_visual(self, rows: T.RowValues, to_left='<', to_up='^',
+                       offset=(0, 0), fmt: T.CellFormat = None):
         """ Merge cells in "visual" mode.
         Symbol: '<' means merged to the left cell, '^' merged to the upper cell.
         E.g.
@@ -261,17 +328,18 @@ class ExcelWriter:
             if len(pos) == 1:
                 self.sheet.write(
                     offset[0] + pos[0][0], offset[1] + pos[0][1], cell,
-                    fmt or self._merge_format
+                    self._convert_format(fmt) or self._merge_format
                 )
             else:
                 self.sheet.merge_range(
                     offset[0] + pos[0][0], offset[1] + pos[0][1],
                     offset[0] + pos[-1][0], offset[1] + pos[-1][1],
-                    cell, cell_format=fmt or self._merge_format
+                    cell, cell_format=(self._convert_format(fmt) or
+                                       self._merge_format)
                 )
     
-    def merging_logical(self, p: Hint.Cell, q: Hint.Cell,
-                        value: Hint.CellValue, fmt=None):
+    def merging_logical(self, p: T.Cell, q: T.Cell, value: T.CellValue,
+                        fmt: T.CellFormat = None):
         """ Merge cells in "logical" mode.
         
         NOTE:
@@ -284,11 +352,11 @@ class ExcelWriter:
             return
         self.sheet.merge_range(
             p[0], p[1], q[0], q[1],
-            value, cell_format=fmt or self._merge_format
+            value, cell_format=self._convert_format(fmt) or self._merge_format
         )
-
+    
     # --------------------------------------------------------------------------
-
+    
     def save(self, alt=False, open_after_saved=False):
         from xlsxwriter.exceptions import FileCreateError
         try:
@@ -314,11 +382,50 @@ class ExcelWriter:
             import os
             os.startfile(os.path.abspath(self.filepath))
         else:
-            from .lk_logger import lk
+            from lk_logger import lk
             lk.logt(
                 '[ExcelWriter][D1139]',
                 f'\n\tExcel saved to "{self.filepath}:0"',
                 h=self.__h
             )
-
+    
     close = save
+
+
+class _SheetManager:
+    
+    def __init__(self):
+        self.current_index = -1
+        
+        self.sheet_by_index = {}  # type: T.Dict[T.Sheetx, T.SheetName]
+        self.sheet_by_name = {}  # type: T.Dict[T.SheetName, T.Sheetx]
+        
+        self.sheet_info = defaultdict(  # type: T.SheetManager
+            lambda: {
+                'sheet_name': '',
+                'sheetx'    : -1,
+                'rowx'      : -1,
+            }
+        )
+    
+    def activate(self, sheetx):
+        self.current_index = sheetx
+    
+    def update_info(self, sheetx: T.Sheetx, info: T.SheetInfo):
+        if sheetx == -1:
+            return
+        self.sheet_by_index[sheetx] = info['sheet_name']
+        self.sheet_by_name[info['sheet_name']] = sheetx
+        # noinspection PyTypeChecker
+        self.sheet_info[sheetx].update(info)
+    
+    def add_new_sheet(self):
+        new_sheetx = len(self.sheet_by_index)
+        return new_sheetx
+    
+    @property
+    def current_name(self) -> T.Optional[T.SheetName]:
+        if self.current_index == -1:
+            return None
+        else:
+            return self.sheet_by_index[self.current_index]
